package com.cultural.heritage.service.common.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.IService;
import com.cultural.heritage.service.common.CommonService;
import org.springframework.stereotype.Service;

import java.lang.reflect.Field;
import java.util.List;
import java.util.function.Function;
import java.util.stream.Collectors;

@Service
public class CommonServiceImpl implements CommonService {


    /**
     * 取出一个集合的某个id列表, 去另外一个集合中筛选出属于这个id列表的集合
     */
    @Override
    public <T, R> List<R> getItemsByIds(List<T> records, IService<R> genericService, Function<T, Long> getId) {
        // 提取ID
        List<Long> ids = records.stream()
                .map(getId)  // 提取每个元素的ID
                .collect(Collectors.toList());

        // 构造查询条件
        QueryWrapper<R> queryWrapper = new QueryWrapper<>();
        queryWrapper.in("id", ids);

        // 返回查询结果
        return genericService.list(queryWrapper);
    }


    /**
     * 根据 ID 列表获取对应的实体列表
     */
    @Override
    public <T> List<T> getListByIds(List<Long> ids, IService<T> genericService) {
        // 构造查询条件
        QueryWrapper<T> queryWrapper = new QueryWrapper<>();
        queryWrapper.in("id", ids);  // 使用 IN 条件

        // 调用具体的服务执行查询
        return genericService.list(queryWrapper);
    }




    /**
     * 从第一个集合中提取某个属性值，并用这些值作为查询条件去查询第二个集合的数据。
     * @param sourceList 原始集合（源数据）
     * @param service 要执行查询的 Service
     * @param sourceField 在原始集合中提取的字段名
     * @param targetField 在目标集合中的查询字段名
     * @param <T> 目标查询实体类型
     * @return 查询结果集合
     */
    @Override
    public <T> List<T> findListByField(List<?> sourceList, IService<T> service, String sourceField, String targetField) {
        // 使用反射获取源集合中对应字段的值
        List<Object> fieldValues = sourceList.stream()
                .map(item -> getFieldValue(item, sourceField)) // 获取字段值
                .collect(Collectors.toList());

        // 如果 fieldValues 为空，直接返回空集合
        if (fieldValues.isEmpty()) {
            return List.of();  // 返回空集合
        }
        // 创建查询条件
        QueryWrapper<T> queryWrapper = new QueryWrapper<>();
        queryWrapper.in(targetField, fieldValues);  // 根据字段值进行查询

        return service.list(queryWrapper);  // 执行查询并返回结果
    }

    /**
     * 使用反射获取对象的字段值
     * @param object 对象
     * @param fieldName 字段名
     * @return 字段的值
     */
    private Object getFieldValue(Object object, String fieldName) {
        try {
            Field field = object.getClass().getDeclaredField(fieldName);
            field.setAccessible(true);
            return field.get(object);
        } catch (NoSuchFieldException | IllegalAccessException e) {
            throw new RuntimeException("字段获取失败", e);
        }
    }


}
